package resourceprompts

import (
	"fmt"

	"github.com/g14a/tf/builder"
	"github.com/g14a/tf/types"
	"github.com/g14a/tf/validators"

	"github.com/fatih/color"
	"github.com/manifoldco/promptui"
)

func AWSAPIGatewayAccountPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "cloudwatch_role_arn",
			Ex:    "",
			Doc: "(Optional) The ARN of an IAM role for CloudWatch (to allow logging & monitoring). " +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/how-to-stage-settings.html#how-to-stage-settings-console " +
				"\nLogging & monitoring can be enabled/disabled and otherwise tuned on the API Gateway Stage level.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_account", blockName, resourceBlock)
}

func AWSAPIGatewayApiKeyPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Required) The name of the API key",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The API key description. Defaults to \"Managed by Terraform\".",
		},
		{
			Field:     "enabled",
			Ex:        "(true/false)",
			Doc:       "(Optional) Specifies whether the API key can be used by callers. Defaults to true.",
			Validator: validators.BoolValidator,
		},
		{
			Field: "value",
			Ex:    "api-key-123",
			Doc:   "(Optional) The value of the API key. If not specified, it will be automatically generated by AWS on creation.",
		},
		{
			Field: "tags",
			Ex:    "k1=v1,k2=v2",
			Doc:   "(Optional) Key-value map of resource tags",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_api_key", blockName, resourceBlock)
}

func AWSAPIGatewayAuthorizerPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "auth-name-123",
			Doc:   "(Required) The name of the authorizer",
		},
		{
			Field: "rest_api_id",
			Ex:    "rai-123",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "identity_source",
			Ex:    "",
			Doc: "(Optional) The source of the identity in an incoming request. Defaults to method.request.header.Authorization. " +
				"\nFor REQUEST type, this may be a comma-separated list of values, including headers, query string parameters and stage variables - e.g. " +
				"\n\"method.request.header.SomeHeaderName,method.request.querystring.SomeQueryStringName,stageVariables.SomeStageVariableName\"",
		},
		{
			Type:  "select",
			Field: "type",
			Doc: "(Optional) The type of the authorizer. Possible values are TOKEN for a " +
				"\nLambda function using a single authorization token submitted in a " +
				"\ncustom header, REQUEST for a Lambda function using incoming request " +
				"\nparameters, or COGNITO_USER_POOLS for using an Amazon Cognito user pool. " +
				"\nDefaults to TOKEN",
			Items: []string{"TOKEN", "REQUEST", "COGNITO_USER_POOLS"},
		},
		{
			Field: "authorizer_credentials",
			Ex:    "",
			Doc: "(Optional) The credentials required for the authorizer. To specify an IAM Role for API Gateway " +
				"\nto assume, use the IAM Role ARN.",
		},
		{
			Field:     "authorizer_result_ttl_in_seconds",
			Ex:        "300",
			Doc:       "(Optional) The TTL of cached authorizer results in seconds. Defaults to 300",
			Validator: validators.IntValidator,
		},
		{
			Field: "identity_validation_expression",
			Ex:    "",
			Doc: "(Optional) A validation expression for the incoming identity. For TOKEN type, " +
				"\nthis value should be a regular expression. The incoming token from the client " +
				"\nis matched against this expression, and will proceed if the token matches. " +
				"\nIf the token doesn't match, the client receives a 401 Unauthorized response.",
		},
		{
			Field: "provider_arns",
			Ex:    "",
			Doc: "(Optional, required for type COGNITO_USER_POOLS) A list of the Amazon Cognito user pool ARNs. " +
				"\nEach element is of this format: arn:aws:cognito-idp:{region}:{account_id}:userpool/{user_pool_id}",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_authorizer", blockName, resourceBlock)
}

func AWSAPIGatewayBasePathMappingPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "domain_name",
			Ex:    "",
			Doc:   "(Required) The already-registered domain name to connect the API to.",
		},
		{
			Field: "api_id",
			Ex:    "",
			Doc:   "(Required) The id of the API to connect.",
		},
		{
			Field: "stage_name",
			Ex:    "",
			Doc: "(Optional) The name of a specific deployment stage to expose at the given path. " +
				"\nIf omitted, callers may select any stage by including its name as a path element after the base path.",
		},
		{
			Field: "base_path",
			Ex:    "",
			Doc: "(Optional) Path segment that must be prepended to the path when accessing the API via this mapping. " +
				"\nIf omitted, the API is exposed at the root of the given domain.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_base_path_mapping", blockName, resourceBlock)
}

func AWSAPIGatewayClientCertificatePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the client certificate.",
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_client_certificate", blockName, resourceBlock)
}

func AWSAPIGatewayDeploymentPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "stage_name",
			Ex:    "",
			Doc: "(Optional) The name of the stage. If the specified stage already exists, it will be updated to " +
				"\npoint to the new deployment. If the stage does not exist, a new one will be created " +
				"\nand point to this deployment.",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the deployment",
		},
		{
			Field: "stage_description",
			Ex:    "",
			Doc:   "(Optional) The description of the stage",
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
		{
			Field:     "variables",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) A map that defines variables for the stage",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_deployment", blockName, resourceBlock)
}

func AWSAPIGatewayDocumentationPartPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "properties",
			Ex:    "",
			Doc: "(Required) A content map of API-specific key-value pairs describing the targeted API entity. " +
				"\nThe map must be encoded as a JSON string, e.g., \"{ \\\"description\\\": \\\"The API does …\\\" }\". " +
				"\nOnly Swagger-compliant key-value pairs can be exported and, hence, published.",
		},
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated Rest API",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Green("\nEnter location:\n(Required) The location of the targeted API entity of the to-be-created documentation part.\n" +
		"The location block supports:" +
		"\n1.method\n2.name\n3.path\n4.status_code\n5.type")

	locationSchema := []types.Schema{
		{
			Field: "method",
			Ex:    "",
			Doc:   "(Optional) The HTTP verb of a method. The default value is * for any method.",
		},
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Optional) The name of the targeted API entity.",
		},
		{
			Field: "path",
			Ex:    "",
			Doc:   "(Optional) The URL path of the target. The default value is / for the root resource.",
		},
		{
			Field: "status_code",
			Ex:    "",
			Doc:   "(Optional) The HTTP status code of a response. The default value is * for any status code.",
		},
		{
			Type:  "select",
			Field: "type",
			Doc:   "(Required) The type of API entity to which the documentation content applies.",
			Items: []string{"API", "METHOD", "REQUEST_BODY"},
		},
	}

	resourceBlock["location"] = builder.PSOrder(types.ProvidePS(locationSchema))

	builder.ResourceBuilder("aws_api_gateway_documentation_part", blockName, resourceBlock)
}

func AWSAPIGatewayDocumentationVersionPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "version",
			Ex:    "",
			Doc:   "(Required) The version identifier of the API documentation snapshot.",
		},
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated Rest API",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the API documentation version.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))
	builder.ResourceBuilder("aws_api_gateway_documentation_version", blockName, resourceBlock)
}

// aws_api_gateway_domain_name
func AWSAPIGatewayDomainNamePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "domain_name",
			Ex:    "",
			Doc:   "(Required) The fully-qualified domain name to register",
		},
		{
			Field: "certificate_arn",
			Ex:    "",
			Doc: "(Optional) The ARN for an AWS-managed certificate. " +
				"\nAWS Certificate Manager is the only supported source. " +
				"\nUsed when an edge-optimized domain name is desired. " +
				"\nConflicts with certificate_name, certificate_body, " +
				"\ncertificate_chain, certificate_private_key, regional_certificate_arn," +
				"\nand regional_certificate_name.",
		},
		{
			Field: "regional_certificate_arn",
			Ex:    "",
			Doc: "(Optional) The ARN for an AWS-managed certificate. " +
				"\nAWS Certificate Manager is the only supported source. " +
				"\nUsed when a regional domain name is desired. Conflicts " +
				"\nwith certificate_arn, certificate_name, certificate_body, " +
				"\ncertificate_chain, and certificate_private_key",
		},
		{
			Field: "certificate_name",
			Ex:    "",
			Doc: "(Optional) The unique name to use when registering this certificate as an IAM server certificate. " +
				"\nConflicts with certificate_arn, regional_certificate_arn, and regional_certificate_name. Required if certificate_arn is not set.",
		},
		{
			Field: "certificate_body",
			Ex:    "",
			Doc: "(Optional) The certificate issued for the domain name being registered, in PEM format. " +
				"\nOnly valid for EDGE endpoint configuration type. Conflicts with certificate_arn, " +
				"\nregional_certificate_arn, and regional_certificate_name",
		},
		{
			Field: "certificate_chain",
			Ex:    "",
			Doc: "(Optional) The certificate for the CA that issued the certificate, along with any intermediate " +
				"\nCA certificates required to create an unbroken chain to a certificate trusted by the intended " +
				"\nAPI clients. Only valid for EDGE endpoint configuration type. Conflicts with certificate_arn, " +
				"\nregional_certificate_arn, and regional_certificate_name",
		},
		{
			Field: "certificate_private_key",
			Ex:    "",
			Doc:   "(Optional) The private key associated with the domain certificate given in certificate_body. Only valid for EDGE endpoint configuration type. Conflicts with certificate_arn, regional_certificate_arn, and regional_certificate_name",
		},
		{
			Field: "regional_certificate_name",
			Ex:    "",
			Doc:   "(Optional) The user-friendly name of the certificate that will be used by regional endpoint for this domain name. Conflicts with certificate_arn, certificate_name, certificate_body, certificate_chain, and certificate_private_key.",
		},
		{
			Type:  "select",
			Field: "security_policy",
			Doc:   "(Optional) The Transport Layer Security (TLS) version + cipher suite for this DomainName. The valid values are TLS_1_0 and TLS_1_2. Must be configured to perform drift detection.",
			Items: []string{"TLS_1_0", "TLS_1_2"},
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Yellow("\nConfigure nested settings like endpoint_configuration/tags [y/n]?\n\n", "text")

	ynPrompt := promptui.Prompt{
		Label: "",
	}

	yn, err := ynPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	if yn == "n" || yn == "" {
		builder.ResourceBuilder("aws_api_gateway_domain_name", blockName, resourceBlock)
		return
	}

	endpointConfigSchema := []types.Schema{
		{
			Field: "types",
			Ex:    "",
			Doc: "(Required) A list of endpoint types. This resource currently only supports " +
				"\nmanaging a single value. Valid values: EDGE or REGIONAL. If unspecified, " +
				"\ndefaults to EDGE. Must be declared as REGIONAL in non-Commercial partitions. " +
				"\nRefer to the documentation for more information on the difference between " +
				"\nedge-optimized and regional APIs.",
			Items: []string{"EDGE", "REGIONAL"},
		},
	}

	resourceBlock["endpoint_configuration"] = builder.PSOrder(types.ProvidePS(endpointConfigSchema))
	builder.ResourceBuilder("aws_api_gateway_domain_name", blockName, resourceBlock)
}

func AWSAPIGatewayGatewayResponsePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The string identifier of the associated REST API.",
		},
		{
			Field: "response_type",
			Ex:    "",
			Doc:   "(Required) The response type of the associated GatewayResponse.",
		},
		{
			Field: "status_code",
			Ex:    "",
			Doc:   "(Optional) The HTTP status code of the Gateway Response.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Green("\nEnter response_templates:\n(Optional) A map specifying the templates used to transform the response body.")

	responseTemplatesSchema := []types.Schema{
		{
			Field: "application/json",
			Ex:    "",
			Doc:   "Check https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/api_gateway_gateway_response",
		},
	}

	resourceBlock["response_templates"] = builder.PSOrder(types.ProvidePS(responseTemplatesSchema))

	builder.ResourceBuilder("aws_api_gateway_account", blockName, resourceBlock)
}

func AWSAPIGatewayIntegrationPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API.",
		},
		{
			Field: "resource_id",
			Ex:    "",
			Doc:   "(Required) The API resource ID.",
		},
		{
			Type:  "select",
			Field: "http_method",
			Doc:   "(Required) The HTTP method",
			Items: []string{"GET", "POST", "PUT", "DELETE", "HEAD", "OPTION", "ANY"},
		},
		{
			Type:  "select",
			Field: "integration_http_method",
			Doc: "(Optional) The integration HTTP method specifying how API Gateway will interact with the back end. " +
				"\nRequired if type is AWS, AWS_PROXY, HTTP or HTTP_PROXY. Not all methods are compatible with all AWS integrations. " +
				"\ne.g. Lambda function can only be invoked via POST." +
				"\nCheckout https://github.com/awslabs/aws-apigateway-importer/issues/9#issuecomment-129651005",
			Items: []string{"GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONs", "ANY", "PATCH"},
		},
		{
			Type:  "select",
			Field: "type",
			Doc: "(Required) The integration input's type. An HTTP or HTTP_PROXY integration with a connection_type " +
				"\nof VPC_LINK is referred to as a private integration and uses a VpcLink to connect API Gateway to a network load balancer of a VPC." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/api-reference/resource/integration/",
			Items: []string{"HTTP", "MOCK", "AWS", "AWS_PROXY", "HTTP_PROXY"},
		},
		{
			Type:  "select",
			Field: "connection_type",
			Doc: "(Optional) The integration input's connectionType. Valid values are INTERNET (default for " +
				"\nconnections through the public routable internet), and VPC_LINK (for private connections " +
				"\nbetween API Gateway and a network load balancer in a VPC)." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/api-reference/resource/integration/#connectionType",
			Items: []string{"INTERNET", "VPC_LINK"},
		},
		{
			Field: "connection_id",
			Ex:    "",
			Doc:   "(Optional) The id of the VpcLink used for the integration. Required if connection_type is VPC_LINK",
		},
		{
			Field: "uri",
			Ex:    "",
			Doc: "(Optional) The input's URI. Required if type is AWS, AWS_PROXY, HTTP or HTTP_PROXY. " +
				"\nFor HTTP integrations, the URI must be a fully formed, encoded HTTP(S) URL according " +
				"\nto the RFC-3986 specification." +
				"\nCheckout https://registry.terraform.io/providers/hashicorp/aws/latest/docs/resources/api_gateway_integration#uri",
		},
		{
			Field: "credentials",
			Ex:    "",
			Doc: "(Optional) The credentials required for the integration. For AWS integrations, 2 options are available. " +
				"\nTo specify an IAM Role for Amazon API Gateway to assume, use the role's ARN. To require that the " +
				"\ncaller's identity be passed through from the request, specify the string arn:aws:iam::\\*:user/\\*",
		},
		{
			Type:  "select",
			Field: "passthrough_behavior",
			Doc:   "(Optional) The integration passthrough behavior",
			Items: []string{"WHEN_NO_MATCH", "WHEN_NO_TEMPLATES", "NEVER"},
		},
		{
			Field: "cache_key_parameters",
			Ex:    "",
			Doc:   "(Optional) A list of cache key parameters for the integration.",
		},
		{
			Field: "cache_namespace",
			Ex:    "",
			Doc:   "(Optional) The integration's cache namespace.",
		},
		{
			Type:  "select",
			Field: "content_handling",
			Doc: "(Optional) Specifies how to handle request payload content type conversions. " +
				"\nIf this property is not defined, the request payload will be passed through " +
				"\nfrom the method request to integration request without modification, provided " +
				"\nthat the passthroughBehaviors is configured to support payload pass-through.",
			Items: []string{"CONVERT_TO_BINARY", "CONVERT_TO_TEXT"},
		},
		{
			Field:     "timeout_milliseconds",
			Ex:        "",
			Doc:       "(Optional) Custom timeout between 50 and 29,000 milliseconds. The default value is 29,000 ms.",
			Validator: validators.MinMaxIntValidator(50, 29000),
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_integration", blockName, resourceBlock)
}

func AWSAPIGatewayIntegrationResponsePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "resource_id",
			Ex:    "",
			Doc:   "(Required) The API resource ID",
		},
		{
			Type:  "select",
			Field: "http_method",
			Doc:   "(Required) The HTTP method",
			Items: []string{"GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS", "ANY"},
		},
		{
			Field: "status_code",
			Ex:    "",
			Doc:   "(Required) The HTTP status code",
		},
		{
			Field: "selection_pattern",
			Ex:    "",
			Doc: "(Optional) Specifies the regular expression pattern used to choose " +
				"\nan integration response based on the response from the backend. " +
				"\nOmit configuring this to make the integration the default one. " +
				"\nIf the backend is an AWS Lambda function, the AWS Lambda function " +
				"\nerror header is matched. For all other HTTP and AWS backends, " +
				"\nthe HTTP status code is matched.",
		},
		{
			Type:  "select",
			Field: "content_handling",
			Doc: "(Optional) Specifies how to handle request payload content type conversions. " +
				"\nSupported values are CONVERT_TO_BINARY and CONVERT_TO_TEXT. If this property " +
				"\nis not defined, the response payload will be passed through from the integration " +
				"\nresponse to the method response without modification.",
			Items: []string{"CONVERT_TO_BINARY", "CONVERT_TO_TEXT"},
		},
		{
			Field:     "response_templates",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) A map specifying the templates used to transform the integration response body",
			Validator: validators.RCValidator,
		},
		{
			Field: "response_parameters",
			Ex:    "k1=v1,k2=v2",
			Doc: "(Optional) A map of response parameters that can be read from the backend response. " +
				"\nFor example: response_parameters = { \"method.response.header.X-Some-Header\" = \"integration.response.header.X-Some-Other-Header\" }",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_integration_response", blockName, resourceBlock)
}

func AWSAPIGatewayMethodPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "resource_id",
			Ex:    "",
			Doc:   "(Required) The API resource ID",
		},
		{
			Type:  "select",
			Field: "http_method",
			Doc:   "(Required) The HTTP method",
			Items: []string{"GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS", "ANY"},
		},
		{
			Type:  "select",
			Field: "authorization",
			Doc:   "(Required) The type of authorization used for the method",
			Items: []string{"NONE", "CUSTOM", "AWS_IAM", "COGNITO_USER_POOLS"},
		},
		{
			Field: "authorizer_id",
			Ex:    "",
			Doc:   "(Optional) The authorizer id to be used when the authorization is CUSTOM or COGNITO_USER_POOLS",
		},
		{
			Field: "authorization_scopes",
			Ex:    "",
			Doc:   "(Optional) The authorization scopes used when the authorization is COGNITO_USER_POOLS",
		},
		{
			Field: "api_key_required",
			Ex:    "",
			Doc:   "(Optional) Specify if the method requires an API key",
		},
		{
			Field: "operation_name",
			Ex:    "",
			Doc: "(Optional) The function name that will be given to the method when generating an " +
				"\nSDK through API Gateway. If omitted, API Gateway will generate a function name " +
				"\nbased on the resource path and HTTP verb.",
		},
		{
			Field: "request_validator_id",
			Ex:    "",
			Doc:   "(Optional) The ID of a aws_api_gateway_request_validator",
		},
		{
			Field: "request_parameters",
			Ex:    "",
			Doc: "(Optional) A map of request parameters (from the path, query string and headers) " +
				"\nthat should be passed to the integration. The boolean value indicates whether the " +
				"\nparameter is required (true) or optional (false). For example: request_parameters = " +
				"\n{\"method.request.header.X-Some-Header\" = true \"method.request.querystring.some-query-param\" = true} " +
				"\nwould define that the header X-Some-Header and the query string some-query-param " +
				"\nmust be provided in the request.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_method", blockName, resourceBlock)
}

func AWSAPIGatewayMethodResponsePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "resource_id",
			Ex:    "",
			Doc:   "(Required) The API resource ID",
		},
		{
			Type:  "select",
			Field: "http_method",
			Doc:   "(Required) The HTTP Method",
			Items: []string{"GET", "POST", "PUT", "DELETE", "HEAD", "OPTIONS", "ANY"},
		},
		{
			Field: "status_code",
			Ex:    "",
			Doc:   "(Required) The HTTP status code",
		},
		{},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_method_response", blockName, resourceBlock)
}

func AWSAPIGatewayMethodSettingsPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the REST API",
		},
		{
			Field: "stage_name",
			Ex:    "",
			Doc:   "(Required) The name of the stage",
		},
		{
			Field: "method_path",
			Ex:    "",
			Doc:   "(Required) Method path defined as {resource_path}/{http_method} for an individual method override, or */* for overriding all methods in the stage.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Yellow("\nConfigure nested settings like settings [y/n]?\n\n", "text")

	ynPrompt := promptui.Prompt{
		Label: "",
	}

	yn, err := ynPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	if yn == "n" || yn == "" {
		builder.ResourceBuilder("aws_api_gateway_method_settings", blockName, resourceBlock)
		return
	}

	color.Green("\nEnter settings:\n(Required) The settings block:\n" +
		"\nThe settings block supports the following:\n" +
		"\n1.metrics_enabled\n2.logging_level\n3.data_trace_enabled\n4.throttling_burst_limit\n5.throttling_late_limit" +
		"\n6.caching_enabled\n7.caching_ttl_in_seconds\n8.cache_data_encrypted\n9.require_authorization_for_cache_control\n10.unauthorized_cache_control_header_strategy")

	settingsSchema := []types.Schema{
		{
			Field:     "metrics_enabled",
			Ex:        "(true/false)",
			Doc:       "(Optional) Specifies whether Amazon CloudWatch metrics are enabled for this method.",
			Validator: validators.BoolValidator,
		},
		{
			Type:  "select",
			Field: "logging_level",
			Doc:   "(Optional) Specifies the logging level for this method, which effects the log entries pushed to Amazon CloudWatch Logs.",
			Items: []string{"OFF", "ERROR", "INFO"},
		},
		{
			Field: "data_trace_enabled",
			Ex:    "(true/false)",
			Doc: "(Optional) Specifies whether data trace logging is enabled for this method, " +
				"\nwhich effects the log entries pushed to Amazon CloudWatch Logs.",
			Validator: validators.BoolValidator,
		},
		{
			Field:     "throttling_burst_limit",
			Ex:        "1",
			Doc:       "(Optional) Specifies the throttling burst limit. Default: -1 (throttling disabled)",
			Validator: validators.IntValidator,
		},
		{
			Field:     "throttling_rate_limit",
			Ex:        "1",
			Doc:       "(Optional) Specifies the throttling rate limit. Default: -1 (throttling disabled)",
			Validator: validators.IntValidator,
		},
		{
			Field: "caching_enabled",
			Ex:    "(true/false)",
			Doc: "(Optional) Specifies whether responses should be cached and returned for requests. " +
				"\nA cache cluster must be enabled on the stage for responses to be cached.",
			Validator: validators.BoolValidator,
		},
		{
			Field: "caching_ttl_in_seconds",
			Ex:    "10",
			Doc: "(Optional) Specifies the time to live (TTL), in seconds, for cached responses. " +
				"\nThe higher the TTL, the longer the response will be cached.",
			Validator: validators.IntValidator,
		},
		{
			Field:     "cache_data_encrypted",
			Ex:        "(true/false)",
			Doc:       "(Optional) Specifies whether the cached responses are encrypted.",
			Validator: validators.BoolValidator,
		},
		{
			Field:     "require_authorization_for_cache_control",
			Ex:        "(true/false)",
			Doc:       "(Optional) Specifies whether authorization is required for a cache invalidation request.",
			Validator: validators.BoolValidator,
		},
		{
			Type:  "select",
			Field: "unauthorized_cache_control_header_strategy",
			Doc:   "(Optional) Specifies how to handle unauthorized requests for cache invalidation.",
			Items: []string{"FAIL_WITH_403", "SUCCEED_WITH_RESPONSE_HEADER", "SUCCEED_WITHOUT_RESPONSE_HEADER"},
		},
	}

	resourceBlock["settings"] = builder.PSOrder(types.ProvidePS(settingsSchema))

	builder.ResourceBuilder("aws_api_gateway_method_settings", blockName, resourceBlock)
}

func AWSAPIGatewayModelPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Required) The name of the model",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the model",
		},
		{
			Field: "content_type",
			Ex:    "",
			Doc:   "(Required) The content type of the model",
		},
		{
			Field: "schema",
			Ex:    "",
			Doc:   "(Required) The schema of the model in a JSON form",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_model", blockName, resourceBlock)
}

func AWSAPIGatewayRequestValidatorPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Required) The name of the request validator",
		},
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated Rest API",
		},
		{
			Field:     "validate_request_body",
			Ex:        "(true/false)",
			Doc:       "(Optional) Boolean whether to validate request body. Defaults to false.",
			Validator: validators.BoolValidator,
		},
		{
			Field:     "validate_request_parameters",
			Ex:        "(true/false)",
			Doc:       "(Optional) Boolean whether to validate request parameters. Defaults to false",
			Validator: validators.BoolValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_request_validator", blockName, resourceBlock)
}

func AWSAPIGatewayResourcePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "parent_id",
			Ex:    "",
			Doc:   "(Required) The ID of the parent API resource",
		},
		{
			Field: "path_part",
			Ex:    "",
			Doc:   "(Required) The last path segment of this API resource.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_resource", blockName, resourceBlock)
}

func AWSAPIGatewayRestAPIPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "",
			Doc: "(Required) Name of the REST API. If importing an OpenAPI specification via the body argument, " +
				"\nthis corresponds to the info.title field. If the argument value is different than the OpenAPI " +
				"\nvalue, the argument value will override the OpenAPI value.",
		},
		{
			Field: "description",
			Ex:    "",
			Doc: "(Optional) Description of the REST API. If importing an OpenAPI specification via the body argument," +
				"\nthis corresponds to the info.description field. If the argument value is provided and is different" +
				"\nthan the OpenAPI value, the argument value will override the OpenAPI value.",
		},
		{
			Field: "binary_media_types",
			Ex:    "[\"t1\",\"t2\"]",
			Doc: "(Optional) List of binary media types supported by the REST API. By default, the REST API supports " +
				"\nonly UTF-8-encoded text payloads. If importing an OpenAPI specification via the body argument, " +
				"\nthis corresponds to the x-amazon-apigateway-binary-media-types extension. If the argument value " +
				"\nis provided and is different than the OpenAPI value, the argument value will override the OpenAPI value.",
		},
		{
			Field: "minimum_compression_size",
			Ex:    "",
			Doc: "(Optional) Minimum response size to compress for the REST API. Integer between -1 and 10485760 (10MB). " +
				"\nSetting a value greater than -1 will enable compression, -1 disables compression (default). " +
				"\nIf importing an OpenAPI specification via the body argument, this corresponds to the x-amazon-apigateway-minimum-compression-size " +
				"\nextension. If the argument value (except -1) is provided and is different than the OpenAPI value, " +
				"\nthe argument value will override the OpenAPI value.",
			Validator: validators.MinMaxIntValidator(-1, 10485760),
		},
		{
			Field: "body",
			Ex:    "",
			Doc: "(Optional) OpenAPI specification that defines the set of routes and integrations to create " +
				"\nas part of the REST API. This configuration, and any updates to it, will replace all REST" +
				"\nAPI configuration except values overridden in this resource configuration and other " +
				"\nresource updates applied after this resource but before any aws_api_gateway_deployment creation." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-import-api.html",
		},
		{
			Field: "parameters",
			Ex:    "",
			Doc: "(Optional) Map of customizations for importing the specification in the body argument. " +
				"\nFor example, to exclude DocumentationParts from an imported API, set ignore equal to documentation." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-import-api.html",
		},
		{
			Field: "policy",
			Ex:    "",
			Doc: "(Optional) JSON formatted policy document that controls access to the API Gateway. " +
				"\nFor more information about building AWS IAM policy documents with Terraform, see the AWS IAM Policy Document Guide. Terraform will only perform drift detection of its value when present in a configuration. It is recommended to use the aws_api_gateway_rest_api_policy resource instead. If importing an OpenAPI specification via the body argument, this corresponds to the x-amazon-apigateway-policy extension. If the argument value is provided and is different than the OpenAPI value, the argument value will override the OpenAPI value.",
		},
		{
			Type:  "select",
			Field: "api_key_source",
			Doc: "(Optional) Source of the API key for requests. If importing an OpenAPI specification " +
				"\nvia the body argument, this corresponds to the x-amazon-apigateway-api-key-source extension. " +
				"\nIf the argument value is provided and is different than the OpenAPI value, the argument " +
				"\nvalue will override the OpenAPI value.",
			Items: []string{"HEADER", "AUTHORIZER"},
		},
		{
			Field: "disable_executable_api_endpoint",
			Ex:    "(true/false)",
			Doc: "(Optional) Specifies whether clients can invoke your API by using the default " +
				"\nexecute-api endpoint. By default, clients can invoke your API with the default " +
				"\nhttps://{api_id}.execute-api.{region}.amazonaws.com endpoint. To require that clients " +
				"\nuse a custom domain name to invoke your API, disable the default endpoint. " +
				"\nDefaults to false. If importing an OpenAPI specification via the body argument, " +
				"\nthis corresponds to the x-amazon-apigateway-endpoint-configuration extension " +
				"\ndisableExecuteApiEndpoint property. If the argument value is true and is different " +
				"\nthan the OpenAPI value, the argument value will override the OpenAPI value.",
			Validator: validators.BoolValidator,
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Yellow("\nConfigure nested settings like endpoint_configuration [y/n]?\n\n", "text")

	ynPrompt := promptui.Prompt{
		Label: "",
	}

	yn, err := ynPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	if yn == "n" || yn == "" {
		builder.ResourceBuilder("aws_api_gateway_rest_api", blockName, resourceBlock)
		return
	}

	endpointConfigurationSchema := []types.Schema{
		{
			Type:  "select",
			Field: "types",
			Doc: "(Required) A list of endpoint types. This resource currently only supports managing a single value. If unspecified, defaults to EDGE. Must be declared as REGIONAL in non-Commercial partitions. Refer to the documentation for more information on the difference between edge-optimized and regional APIs." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/create-regional-api.html",
			Items: []string{"EDGE", "RELATIONAL", "PRIVATE"},
		},
		{
			Field: "vpc_endpoint_ids",
			Ex:    "",
			Doc: "(Optional) Set of VPC Endpoint identifiers. It is only supported for PRIVATE endpoint type. If importing an OpenAPI specification via the body argument, this corresponds to the x-amazon-apigateway-endpoint-configuration extension vpcEndpointIds property. If the argument value is provided and is different than the OpenAPI value, the argument value will override the OpenAPI value." +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/api-gateway-swagger-extensions-endpoint-configuration.html",
		},
	}

	resourceBlock["endpoint_configuration"] = builder.PSOrder(types.ProvidePS(endpointConfigurationSchema))

	builder.ResourceBuilder("aws_api_gateway_rest_api", blockName, resourceBlock)
}

func AWSAPIGatewayRestAPIPolicyPrompt() {

}

func AWSAPIGatewayStagePrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "rest_api_id",
			Ex:    "",
			Doc:   "(Required) The ID of the associated REST API",
		},
		{
			Field: "stage_name",
			Ex:    "",
			Doc:   "(Required) The name of the stage",
		},
		{
			Field: "deployment_id",
			Ex:    "",
			Doc:   "(Required) The ID of the deployment that the stage points to",
		},
		{
			Field:     "cache_cluster_enabled",
			Ex:        "(true/false)",
			Doc:       "(Optional) Specifies whether a cache cluster is enabled for the stage",
			Validator: validators.BoolValidator,
		},
		{
			Type:  "select",
			Field: "cache_cluster_size",
			Doc:   "(Optional) The size of the cache cluster for the stage, if enabled.",
			Items: []string{"0.5", "1.6", "6.1", "13.5", "28.4", "58.2", "118", "237"},
		},
		{
			Field: "client_certificate_id",
			Ex:    "",
			Doc:   "(Optional) The identifier of a client certificate for the stage.",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the stage",
		},
		{
			Field: "documentation_version",
			Ex:    "",
			Doc:   "(Optional) The version of the associated API documentation",
		},
		{
			Field:     "variable",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) A map that defines the stage variables",
			Validator: validators.RCValidator,
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) A map of tags to assign to the resource.",
			Validator: validators.RCValidator,
		},
		{
			Field:     "xray_tracing_enabled",
			Ex:        "(true/false)",
			Doc:       "(Optional) Whether active tracing with X-ray is enabled. Defaults to false.",
			Validator: validators.BoolValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Yellow("\nConfigure nested settings like access_log_settings [y/n]?\n\n", "text")

	ynPrompt := promptui.Prompt{
		Label: "",
	}

	yn, err := ynPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	if yn == "n" || yn == "" {
		builder.ResourceBuilder("aws_api_gateway_stage", blockName, resourceBlock)
		return
	}

	accessLogSettingsSchema := []types.Schema{
		{
			Field: "destination_arn",
			Ex:    "",
			Doc:   "(Required) The Amazon Resource Name (ARN) of the CloudWatch Logs log group or Kinesis Data Firehose delivery stream to receive access logs. If you specify a Kinesis Data Firehose delivery stream, the stream name must begin with amazon-apigateway-. Automatically removes trailing :* if present.",
		},
		{
			Field: "format",
			Ex:    "",
			Doc: "(Required) The formatting and values recorded in the logs. " +
				"\nCheckout https://docs.aws.amazon.com/apigateway/latest/developerguide/set-up-logging.html",
		},
	}

	resourceBlock["access_log_settings"] = builder.PSOrder(types.ProvidePS(accessLogSettingsSchema))

	builder.ResourceBuilder("aws_api_gateway_stage", blockName, resourceBlock)
}

func AWSAPIGatewayUsagePlanPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Required) The name of the usage plan.",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of a usage plan.",
		},
		{
			Field: "product_code",
			Ex:    "",
			Doc:   "(Optional) The AWS Marketplace product identifier to associate with the usage plan as a SaaS product on AWS Marketplace.",
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	color.Yellow("\nConfigure nested settings like quota_settings/throttle_settings [y/n]?\n\n", "text")

	ynPrompt := promptui.Prompt{
		Label: "",
	}

	yn, err := ynPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	if yn == "n" || yn == "" {
		builder.ResourceBuilder("aws_api_gateway_usage_plan", blockName, resourceBlock)
		return
	}

	color.Green("\nEnter api_stages:\n(Optional) The associated API stages of the usage plan." +
		"\nThe api_stages block supports the following:" +
		"\n1.api_id\n2.stage\n")

	apiStagesSchema := []types.Schema{
		{
			Field: "api_id",
			Ex:    "",
			Doc:   "(Required) - API Id of the associated API stage in a usage plan.",
		},
		{
			Field: "stages",
			Ex:    "",
			Doc:   "(Required) - API stage name of the associated API stage in a usage plan.",
		},
	}

	resourceBlock["api_stages"] = builder.PSOrder(types.ProvidePS(apiStagesSchema))

	color.Green("\nEnter quota_settings:\n(Optional) The quota settings of the usage plan." +
		"\nThe quota_settings block supports the following:" +
		"\n1.limit\n2.offset\n3.period:\n")

	quotaSettingsSchema := []types.Schema{
		{
			Field:     "limit",
			Ex:        "100",
			Doc:       "(Optional) - The maximum number of requests that can be made in a given time period.",
			Validator: validators.IntValidator,
		},
		{
			Field:     "offset",
			Ex:        "100",
			Doc:       "(Optional) - The number of requests subtracted from the given limit in the initial time period.",
			Validator: validators.IntValidator,
		},
		{
			Type:  "select",
			Field: "period",
			Doc:   "(Optional) - The time period in which the limit applies.",
			Items: []string{"DAY", "WEEK", "MONTH"},
		},
	}

	resourceBlock["quota_settings"] = builder.PSOrder(types.ProvidePS(quotaSettingsSchema))

	color.Green("\nEnter throttle_settings:\n(Optional) The throttling limits of the usage plan." +
		"\nThe quota_settings block supports the following:" +
		"\n1.burst_limit\n2.rate_limit\n")

	throttleSettingsSchema := []types.Schema{
		{
			Field:     "burst_limit",
			Ex:        "100",
			Doc:       "(Optional) - The API request burst limit, the maximum rate limit over a time ranging from one to a few seconds, depending upon whether the underlying token bucket is at its full capacity.",
			Validator: validators.IntValidator,
		},
		{
			Field:     "rate_limit",
			Ex:        "100",
			Doc:       "(Optional) - The API request steady-state rate limit.",
			Validator: validators.IntValidator,
		},
	}

	resourceBlock["throttle_settings"] = builder.PSOrder(types.ProvidePS(throttleSettingsSchema))

	builder.ResourceBuilder("aws_api_gateway_usage_plan", blockName, resourceBlock)
}

func AWSAPIGatewayUsagePlanKeyPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "key_id",
			Ex:    "",
			Doc:   "(Required) The identifier of the API key resource.",
		},
		{
			Type:  "select",
			Field: "key_type",
			Doc:   "(Required) The type of the API key resource. Currently, the valid key type is API_KEY.",
			Items: []string{"API_KEY"},
		},
		{
			Field: "usage_plan_id",
			Ex:    "",
			Doc:   "(Required) The Id of the usage plan resource representing to associate the key to.",
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_usage_plan_key", blockName, resourceBlock)
}

func AWSAPIGatewayVPCLinkPrompt() {
	color.Green("\nEnter block name(Required) e.g. foo/bar\n\n")
	blockPrompt := promptui.Prompt{
		Label: "",
	}

	blockName, err := blockPrompt.Run()
	if err != nil {
		fmt.Println(err)
	}

	schema := []types.Schema{
		{
			Field: "name",
			Ex:    "",
			Doc:   "(Required) The name used to label and identify the VPC link.",
		},
		{
			Field: "description",
			Ex:    "",
			Doc:   "(Optional) The description of the VPC link.",
		},
		{
			Field: "target_arns",
			Ex:    "",
			Doc:   "(Required, ForceNew) The list of network load balancer arns in the VPC targeted by the VPC link. Currently AWS only supports 1 target.",
		},
		{
			Field:     "tags",
			Ex:        "k1=v1,k2=v2",
			Doc:       "(Optional) Key-value map of resource tags",
			Validator: validators.RCValidator,
		},
	}

	resourceBlock := builder.PSOrder(types.ProvidePS(schema))

	builder.ResourceBuilder("aws_api_gateway_vpc_link", blockName, resourceBlock)
}
